clear all;close all;clc
%% Load Raw Data For the Case Study: Flow around Chlamydomonas reinhardtii

load('CaseStudy Raw Data\Raw_Signals_FlowAroundChlamydomonas.mat')
%% Add EKF/UKF Toolbox functions
addpath('ekfukf')
%% Parameters of the OTV
fprintf('The trap stiffness in x-direction is %f [pN/micron] \n',stiff_x*1e6)
fprintf('The trap stiffness in y-direction is %f [pN/micron] \n',stiff_y*1e6)
fprintf('The bead radius is %f [micron]\n',beadsize/2)

Fs=10000;dt=1/Fs;
gamma=6*pi*0.9544e-3*beadsize*0.5e-6 ;% Stokes Drag coefficient: gamma=6\pi\mu a

A00_y=1-stiff_y*dt/gamma; % Defined in the paper
A00_x=1-stiff_x*dt/gamma; % Defined in the paper

H00=1; % Defined in the paper

freq=[0 51.2 95.8 102.2 100.1 145 151 157.4 200 207.8];  % Choose the central frequencies to expand around

order=2; % Choose the Taylor expansion order

y(6000:end)=[]; % Look only on  first 0.6 seconds part of measurements
x(6000:end)=[]; % Look only on  first 0.6 seconds part of measurements

%%% The noise magnitudes for the measurement and process noises
delta_x2 = (2*pi)^2*1.38e-23*295/(stiff_y);
R = delta_x2*30;
Q_diag=[delta_x2*1/100,repmat(delta_x2*1000*repmat([1 1*ones(1,order)],1,length(freq)),1,2)];

% Taylor Kalman filter with 2nd order expantion in y-direction
[y_filt,u_y] = Tylor_Kalmn(y,Fs,A00_y,H00,Q_diag,R,freq,order);

% Taylor Kalman filter with 2nd order expantion in x-direction
[x_filt,u_x] = Tylor_Kalmn(x,Fs,A00_x,H00,Q_diag,R,freq,order);

%% Check figure: see the spectrum of filtered noise and the raw signal

fig=figure('Units','inches',...
    'Position',[4 3 2*3.4 2.8],...
    'PaperPositionMode','auto');

subplot(121)
[f,pow]=fourier_spectrum(y,Fs);
loglog(f,pow,'-','Linewidth',2,'Color','b')
hold on
[f,pow]=fourier_spectrum(y-y_filt',Fs);
loglog(f,pow,'-','Linewidth',2,'Color','r')
xlabel('f [Hz]','FontUnits','points',...
    'interpreter','latex',...
    'FontSize',12,...
    'FontName','Times')
ylabel('Amplitude [nm]','FontUnits','points',...
    'interpreter','latex',...
    'FontSize',12,...
    'FontName','Times')
set(gca,'FontSize',12)
xlim([1 5000])
legend('Measurement: y(t)','Filtered Noise','Location','southwest','Box','off','Interpreter','latex')

subplot(122)
[f,pow]=fourier_spectrum(x,Fs);
loglog(f,pow,'-','Linewidth',2,'Color','b')
hold on
[f,pow]=fourier_spectrum(x-x_filt',Fs);
loglog(f,pow,'-','Linewidth',2,'Color','r')
xlabel('f [Hz]','FontUnits','points',...
    'interpreter','latex',...
    'FontSize',12,...
    'FontName','Times')
ylabel('Amplitude [nm]','FontUnits','points',...
    'interpreter','latex',...
    'FontSize',12,...
    'FontName','Times')
set(gca,'FontSize',12)
xlim([1 5000])
legend('Measurement: x(t)','Filtered Noise','Location','southwest','Box','off','Interpreter','latex')

%% Plot Reslts

CaseStudy_Plot_Signals_TaylorKalman

