clear all;close all;clc

%% Add EKF/UKF Toolbox functions
addpath('ekfukf')
%% Load Raw Data
Amp = 7; % There are seven different amplitudes of flow, 
               % 1 corresponds to the smallest, and 7 corresponds 
               % to the largest amplitude
Rep = 1; % there are 9 repetitions of each expriment

load(['raw Data of 50Hz flows\', 'x(t)_Amplitude_', num2str(Amp),...
        '_Repetition_',  num2str(Rep),'.mat']);
%% Parameters of the OTV
fprintf('The trap stiffness is %f [pN/micron] \n',stiff*1e6)
fprintf('The bead radius is %f [micron]\n',beadsize/2)

dt=1/Fs;
Nt = length(y);

gamma=6*pi*0.9544e-3*beadsize*0.5e-6; % Stokes Drag coefficient: gamma=6\pi\mu a

A00=1-stiff*dt/gamma; % Defined in the paper

H00=1; % Defined in the paper
%% Parameters of the Taylor-Kalman Filter
freq=[48 49.45 50.5]; % Choose the central frequencies to expand around
order=2; % Choose the Taylor expansion order

%%% The noise magnitudes for the measurement and process noises
delta_x2 = (2*pi)^2*1.38e-23*295/(stiff);
R = delta_x2*500;
Q_diag=[delta_x2*1/3,repmat(delta_x2*50*repmat([1 1*ones(1,order)],1,length(freq)),1,2)];

[y_filt,u_input] = Tylor_Kalmn(y,Fs,A00,H00,Q_diag,R,freq,order);

u_otv = u_input;
xf_otv = y_filt;

%% Check figure: see the spectrum of filtered noise and the raw signal
figure
y = y-mean(y);

[f,pow]=fourier_spectrum(y,Fs);
loglog(f,pow,'-','Linewidth',2,'Color','b')
hold on
[f,pow]=fourier_spectrum(y-y_filt',Fs);
loglog(f,pow,'-','Linewidth',2,'Color','r')
xlabel('f [Hz]','FontUnits','points',...
    'interpreter','latex',...
    'FontSize',12,...
    'FontName','Times')
ylabel('Amplitude [nm]','FontUnits','points',...
    'interpreter','latex',...
    'FontSize',12,...
    'FontName','Times')
set(gca,'FontSize',12)
xlim([1 5000])
legend('Measurement','Filtered Noise','Orientation','horizontal','Location','southwest','Box','off','Interpreter','latex')

%% Plot signals
Plot_signals_TaylorKalman
